@@ -5,7 +5,13 @@ related utilities used by hledger commands.
 
 -}
 
-{-# LANGUAGE CPP, ScopedTypeVariables, FlexibleContexts, TypeFamilies, OverloadedStrings, PackageImports #-}
+{-# LANGUAGE CPP                 #-}
+{-# LANGUAGE FlexibleContexts    #-}
+{-# LANGUAGE OverloadedStrings   #-}
+{-# LANGUAGE PackageImports      #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TemplateHaskell     #-}
+{-# LANGUAGE TypeFamilies        #-}
 
 module Hledger.Cli.CliOptions (
 
@@ -34,14 +40,13 @@ module Hledger.Cli.CliOptions (
 
   -- * CLI options
   CliOpts(..),
+  HasCliOpts(..),
   defcliopts,
   getHledgerCliOpts,
   getHledgerCliOpts',
   rawOptsToCliOpts,
-  checkCliOpts,
   outputFormats,
   defaultOutputFormat,
-  defaultBalanceLineFormat,
   CommandDoc,
 
   -- possibly these should move into argsToCliOpts
@@ -56,8 +61,6 @@ module Hledger.Cli.CliOptions (
   replaceNumericFlags,
   -- | For register:
   registerWidthsFromOpts,
-  -- | For balance:
-  lineFormatFromOpts,
 
   -- * Other utils
   hledgerAddons,
@@ -69,18 +72,14 @@ module Hledger.Cli.CliOptions (
 )
 where
 
-import Prelude ()
-import "base-compat-batteries" Prelude.Compat
 import qualified Control.Exception as C
 import Control.Monad (when)
 import Data.Char
 import Data.Default
-import Data.Either (isRight)
+import Data.Either (fromRight, isRight)
 import Data.Functor.Identity (Identity)
-import "base-compat-batteries" Data.List.Compat
-import Data.List.Extra (nubSort)
+import Data.List.Extra (groupSortOn, intercalate, isInfixOf, nubSort)
 import Data.List.Split (splitOneOf)
-import Data.Ord
 import Data.Maybe
 --import Data.String.Here
 -- import Data.Text (Text)
@@ -103,6 +102,7 @@ import Text.Megaparsec.Char
 import Hledger
 import Hledger.Cli.DocFiles
 import Hledger.Cli.Version
+import Data.Time.Clock.POSIX (POSIXTime)
 
 
 -- common cmdargs flags
@@ -110,9 +110,12 @@ import Hledger.Cli.Version
 -- | Common help flags: --help, --debug, --version...
 helpflags :: [Flag RawOpts]
 helpflags = [
-  flagNone ["help","h"] (setboolopt "help") "show general usage (or after CMD, command usage)"
+  -- XXX why are these duplicated in defCommandMode below ?
+  flagNone ["help","h"] (setboolopt "help") "show general help (or after CMD, command help)"
+ ,flagNone ["man"] (setboolopt "man") "Show user manual with man"
+ ,flagNone ["info"] (setboolopt "info") "Show info manual with info"
  -- ,flagNone ["browse-args"] (setboolopt "browse-args") "use a web UI to select options and build up a command line"
- ,flagReq  ["debug"]    (\s opts -> Right $ setopt "debug" s opts) "[N]" "show debug output (levels 1-9, default: 1)"
+ ,flagReq  ["debug"] (\s opts -> Right $ setopt "debug" s opts) "[N]" "show debug output (levels 1-9, default: 1)"
  ,flagNone ["version"] (setboolopt "version") "show version information"
  ]
 
@@ -129,6 +132,7 @@ inputflags = [
  ,flagNone ["anon"]          (setboolopt "anon") "anonymize accounts and payees"
  ,flagReq  ["pivot"]         (\s opts -> Right $ setopt "pivot" s opts)  "TAGNAME" "use some other field/tag for account names"
  ,flagNone ["ignore-assertions","I"] (setboolopt "ignore-assertions") "ignore any balance assertions"
+ ,flagNone ["strict","s"]    (setboolopt "strict") "do extra error checking (check that all posted accounts are declared)"
  ]
 
 -- | Common report-related flags: --period, --cost, etc.
@@ -136,8 +140,8 @@ reportflags :: [Flag RawOpts]
 reportflags = [
 
   -- report period & interval
-  flagReq  ["begin","b"]     (\s opts -> Right $ setopt "begin" s opts) "DATE" "include postings/txns on or after this date"
- ,flagReq  ["end","e"]       (\s opts -> Right $ setopt "end" s opts) "DATE" "include postings/txns before this date"
+  flagReq  ["begin","b"]     (\s opts -> Right $ setopt "begin" s opts) "DATE" "include postings/txns on or after this date (will be adjusted to preceding subperiod start when using a report interval)"
+ ,flagReq  ["end","e"]       (\s opts -> Right $ setopt "end" s opts) "DATE" "include postings/txns before this date (will be adjusted to following subperiod end when using a report interval)"
  ,flagNone ["daily","D"]     (setboolopt "daily") "multiperiod/multicolumn report by day"
  ,flagNone ["weekly","W"]    (setboolopt "weekly") "multiperiod/multicolumn report by week"
  ,flagNone ["monthly","M"]   (setboolopt "monthly") "multiperiod/multicolumn report by month"
@@ -145,7 +149,8 @@ reportflags = [
  ,flagNone ["yearly","Y"]    (setboolopt "yearly") "multiperiod/multicolumn report by year"
  ,flagReq  ["period","p"]    (\s opts -> Right $ setopt "period" s opts) "PERIODEXP" "set start date, end date, and/or report interval all at once"
  ,flagNone ["date2"]         (setboolopt "date2") "match the secondary date instead. See command help for other effects. (--effective, --aux-date also accepted)"  -- see also hiddenflags
-
+ ,flagReq  ["today"]         (\s opts -> Right $ setopt "today" s opts) "DATE" "override today's date (affects relative smart dates, for tests/examples)"
+ 
   -- status/realness/depth/zero filters
  ,flagNone ["unmarked","U"]  (setboolopt "unmarked") "include only unmarked postings/txns (can combine with -P or -C)"
  ,flagNone ["pending","P"]   (setboolopt "pending") "include only pending postings/txns"
@@ -156,29 +161,41 @@ reportflags = [
 
   -- valuation
  ,flagNone ["B","cost"]      (setboolopt "B")
-   "show amounts converted to their cost/selling amount, using the transaction price. Equivalent to --value=cost."
+   "show amounts converted to their cost/selling amount, using the transaction price."
  ,flagNone ["V","market"]    (setboolopt "V")
    (unwords
-     ["show amounts converted to current market value (single period reports)"
-     ,"or period-end market value (multiperiod reports) in their default valuation commodity."
-     ,"Equivalent to --value=now / --value=end."
+     ["show amounts converted to period-end market value in their default valuation commodity."
+     ,"Equivalent to --value=end."
      ])
  ,flagReq ["X","exchange"]   (\s opts -> Right $ setopt "X" s opts) "COMM"
    (unwords
      ["show amounts converted to current (single period reports)"
      ,"or period-end (multiperiod reports) market value in the specified commodity."
-     ,"Equivalent to --value=now,COMM / --value=end,COMM."
+     ,"Equivalent to --value=end,COMM."
      ])
  ,flagReq  ["value"]         (\s opts -> Right $ setopt "value" s opts) "TYPE[,COMM]"
    (unlines
      ["show amounts converted with valuation TYPE, and optionally to specified commodity COMM. TYPE can be:"
-     ,"'cost': convert to cost using transaction prices, then optionally to COMM using period-end market prices"
      ,"'then': convert to contemporaneous market value, in default valuation commodity or COMM (print & register commands only)"
      ,"'end':  convert to period-end market value, in default valuation commodity or COMM"
      ,"'now':  convert to current market value, in default valuation commodity or COMM"
      ,"YYYY-MM-DD: convert to market value on the given date, in default valuation commodity or COMM"
      ])
- ,flagNone ["infer-value"]   (setboolopt "infer-value") "with -V/-X/--value, also infer market prices from transactions"
+  ,flagNone ["infer-equity"] (setboolopt "infer-equity")
+    "in conversion transactions, replace costs (transaction prices) with equity postings, to keep the transactions balanced"
+  
+  -- history of this flag so far, lest we be confused:
+  --  originally --infer-value
+  --  2021-02 --infer-market-price added, --infer-value deprecated 
+  --  2021-09
+  --   --infer-value hidden
+  --   --infer-market-price renamed to --infer-market-prices, old spelling still works
+  --   ReportOptions{infer_value_} renamed to infer_prices_, BalancingOpts{infer_prices_} renamed to infer_transaction_prices_
+  --   some related prices command changes
+  --    --costs deprecated and hidden, uses --infer-market-prices instead
+  --    --inverted-costs renamed to --infer-reverse-prices
+ ,flagNone ["infer-market-prices"] (setboolopt "infer-market-prices") 
+    "use transaction prices (recorded with @ or @@) as additional market prices, as if they were P directives"
 
   -- generated postings/transactions
  ,flagNone ["auto"]          (setboolopt "auto") "apply automated posting rules to modify transactions"
@@ -190,7 +207,10 @@ reportflags = [
    ])
 
   -- general output-related
-
+ ,flagReq ["commodity-style", "c"] (\s opts -> Right $ setopt "commodity-style" s opts) "COMM"
+    "Override the commodity style in the output for the specified commodity. For example 'EUR1.000,00'."
+  
+  -- This has special support in hledger-lib:colorOption, keep synced
  ,flagReq  ["color","colour"] (\s opts -> Right $ setopt "color" s opts) "WHEN"
    (unlines
      ["Should color-supporting commands use ANSI color codes in text output."
@@ -199,6 +219,12 @@ reportflags = [
      ,"'never' or 'no': never."
      ,"A NO_COLOR environment variable overrides this."
      ])
+ ,flagOpt "yes" ["pretty"] (\s opts -> Right $ setopt "pretty" s opts) "WHEN"
+   (unlines
+     ["Show prettier output, e.g. using unicode box-drawing characters."
+     ,"Accepts 'yes' (the default) or 'no'."
+     ,"If you provide an argument you must use '=', e.g. '--pretty=yes'."
+     ])
  ]
 
 -- | Flags for selecting flat/tree mode, used for reports organised by account.
@@ -216,8 +242,10 @@ flattreeflags showamounthelp = [
 -- such as --effective, --aux-date.
 hiddenflags :: [Flag RawOpts]
 hiddenflags = [
-  flagNone ["effective","aux-date"] (setboolopt "date2") "Ledger-compatible aliases for --date2"
- ]
+   flagNone ["effective","aux-date"] (setboolopt "date2") "Ledger-compatible aliases for --date2"
+  ,flagNone ["infer-value"] (setboolopt "infer-market-prices") "legacy flag that was renamed"
+  ,flagNone ["pretty-tables"] (setopt "pretty" "always") "legacy flag that was renamed"
+  ]
 
 -- | Common output-related flags: --output-file, --output-format...
 
@@ -228,13 +256,14 @@ outputFormatFlag fmts = flagReq
   ["output-format","O"] (\s opts -> Right $ setopt "output-format" s opts) "FMT"
   ("select the output format. Supported formats:\n" ++ intercalate ", " fmts ++ ".")
 
+-- This has special support in hledger-lib:outputFileOption, keep synced
 outputFileFlag :: Flag RawOpts
 outputFileFlag = flagReq
   ["output-file","o"] (\s opts -> Right $ setopt "output-file" s opts) "FILE"
   "write output to FILE. A file extension matching one of the above formats selects that format."
 
 argsFlag :: FlagHelp -> Arg RawOpts
-argsFlag desc = flagArg (\s opts -> Right $ setopt "args" s opts) desc
+argsFlag = flagArg (\s opts -> Right $ setopt "args" s opts)
 
 generalflagstitle :: String
 generalflagstitle = "\nGeneral flags"
@@ -277,8 +306,10 @@ defCommandMode names = defMode {
   ,modeGroupFlags  = Group {
      groupNamed   = []
     ,groupUnnamed = [
-       flagNone ["help"] (setboolopt "help") "Show usage."
+        flagNone ["help"] (setboolopt "help") "Show command-line help"
       -- ,flagNone ["help"] (setboolopt "help") "Show long help."
+       ,flagNone ["man"] (setboolopt "man") "Show user manual with man"
+       ,flagNone ["info"] (setboolopt "info") "Show info manual with info"
       ]
     ,groupHidden  = []             --  flags not displayed in the usage
     }
@@ -403,7 +434,7 @@ data CliOpts = CliOpts {
     ,command_         :: String
     ,file_            :: [FilePath]
     ,inputopts_       :: InputOpts
-    ,reportopts_      :: ReportOpts
+    ,reportspec_      :: ReportSpec
     ,output_file_     :: Maybe FilePath
     ,output_format_   :: Maybe String
     ,debug_           :: Int            -- ^ debug level, set by @--debug[=N]@. See also 'Hledger.Utils.debugLevel'.
@@ -413,23 +444,26 @@ data CliOpts = CliOpts {
                                         -- 1. the COLUMNS env var, if set
                                         -- 2. the width reported by the terminal, if supported
                                         -- 3. the default (80)
+    ,progstarttime_   :: POSIXTime
  } deriving (Show)
 
 instance Default CliOpts where def = defcliopts
 
 defcliopts :: CliOpts
 defcliopts = CliOpts
-    def
-    def
-    def
-    def
-    def
-    def
-    def
-    def
-    def
-    def
-    defaultWidth
+    { rawopts_         = def
+    , command_         = ""
+    , file_            = []
+    , inputopts_       = definputopts
+    , reportspec_      = def
+    , output_file_     = Nothing
+    , output_format_   = Nothing
+    , debug_           = 0
+    , no_new_accounts_ = False
+    , width_           = Nothing
+    , available_width_ = defaultWidth
+    , progstarttime_   = 0
+    }
 
 -- | Default width for hledger console output, when not otherwise specified.
 defaultWidth :: Int
@@ -448,15 +482,20 @@ replaceNumericFlags = map replace
 -- today's date. Parsing failures will raise an error.
 -- Also records the terminal width, if supported.
 rawOptsToCliOpts :: RawOpts -> IO CliOpts
-rawOptsToCliOpts rawopts = checkCliOpts <$> do
-  let iopts = rawOptsToInputOpts rawopts
-  ropts <- rawOptsToReportOpts rawopts
+rawOptsToCliOpts rawopts = do
+  currentDay <- getCurrentDay
+  let day = case maybestringopt "today" rawopts of
+              Nothing -> currentDay
+              Just d  -> fromRight (error' $ "Unable to parse date \"" ++ d ++ "\"") -- PARTIAL:
+                         $ fixSmartDateStrEither' currentDay (T.pack d)
+  let iopts = rawOptsToInputOpts day rawopts
+  rspec <- either error' pure $ rawOptsToReportSpec day rawopts  -- PARTIAL:
   mcolumns <- readMay <$> getEnvSafe "COLUMNS"
   mtermwidth <-
 #ifdef mingw32_HOST_OS
     return Nothing
 #else
-    setupTermFromEnv >>= return . flip getCapability termColumns
+    (`getCapability` termColumns) <$> setupTermFromEnv
     -- XXX Throws a SetupTermError if the terminfo database could not be read, should catch
 #endif
   let availablewidth = head $ catMaybes [mcolumns, mtermwidth, Just defaultWidth]
@@ -465,7 +504,7 @@ rawOptsToCliOpts rawopts = checkCliOpts <$> do
              ,command_         = stringopt "command" rawopts
              ,file_            = listofstringopt "file" rawopts
              ,inputopts_       = iopts
-             ,reportopts_      = ropts
+             ,reportspec_      = rspec
              ,output_file_     = maybestringopt "output-file" rawopts
              ,output_format_   = maybestringopt "output-format" rawopts
              ,debug_           = posintopt "debug" rawopts
@@ -474,16 +513,6 @@ rawOptsToCliOpts rawopts = checkCliOpts <$> do
              ,available_width_ = availablewidth
              }
 
--- | Do final validation of processed opts, raising an error if there is trouble.
-checkCliOpts :: CliOpts -> CliOpts
-checkCliOpts opts =
-  either usageError (const opts) $ do
-    -- XXX move to checkReportOpts or move _format to CliOpts
-    case lineFormatFromOpts $ reportopts_ opts of
-      Left err -> Left $ "could not parse format option: "++err
-      Right _  -> Right ()
-  -- XXX check registerWidthsFromOpts opts
-
 -- | A helper for addon commands: this parses options and arguments from
 -- the current command line using the given hledger-style cmdargs mode,
 -- and returns a CliOpts. Or, with --help or -h present, it prints
@@ -532,8 +561,7 @@ getHledgerCliOpts' mode' args' = do
         putStrLn $ "running: " ++ progname'
         putStrLn $ "raw args: " ++ show args'
         putStrLn $ "processed opts:\n" ++ show opts
-        d <- getCurrentDay
-        putStrLn $ "search query: " ++ show (queryFromOpts d $ reportopts_ opts)
+        putStrLn $ "search query: " ++ show (_rsQuery $ reportspec_ opts)
 
 getHledgerCliOpts :: Mode RawOpts -> IO CliOpts
 getHledgerCliOpts mode' = do
@@ -563,22 +591,20 @@ expandPathPreservingPrefix d prefixedf = do
     Just p  -> p ++ ":" ++ f'
     Nothing -> f'
 
--- | Get the expanded, absolute output file path from options,
--- or the default (-, meaning stdout).
-outputFileFromOpts :: CliOpts -> IO FilePath
+-- | Get the expanded, absolute output file path specified by an
+-- -o/--output-file options, or nothing, meaning stdout.
+outputFileFromOpts :: CliOpts -> IO (Maybe FilePath)
 outputFileFromOpts opts = do
   d <- getCurrentDirectory
   case output_file_ opts of
-    Just p  -> expandPath d p
-    Nothing -> return "-"
+    Nothing -> return Nothing
+    Just f  -> Just <$> expandPath d f
 
+defaultOutputFormat :: String
 defaultOutputFormat = "txt"
 
-outputFormats =
-  [defaultOutputFormat] ++
-  ["csv"
-  ,"html"
-  ]
+outputFormats :: [String]
+outputFormats = [defaultOutputFormat, "csv", "html"]
 
 -- | Get the output format from the --output-format option,
 -- otherwise from a recognised file extension in the --output-file option,
@@ -643,22 +669,6 @@ registerWidthsFromOpts CliOpts{width_=Just s}  =
           eof
           return (totalwidth, descwidth)
 
--- for balance, currently:
-
--- | Parse the format option if provided, possibly returning an error,
--- otherwise get the default value.
-lineFormatFromOpts :: ReportOpts -> Either String StringFormat
-lineFormatFromOpts = maybe (Right defaultBalanceLineFormat) parseStringFormat . format_
-
--- | Default line format for balance report: "%20(total)  %2(depth_spacer)%-(account)"
-defaultBalanceLineFormat :: StringFormat
-defaultBalanceLineFormat = BottomAligned [
-      FormatField False (Just 20) Nothing TotalField
-    , FormatLiteral "  "
-    , FormatField True (Just 2) Nothing DepthSpacerField
-    , FormatField True Nothing Nothing AccountField
-    ]
-
 -- Other utils
 
 -- | Get the sorted unique canonical names of hledger addon commands
@@ -680,12 +690,11 @@ defaultBalanceLineFormat = BottomAligned [
 hledgerAddons :: IO [String]
 hledgerAddons = do
   -- past bug generator
-  as1 <- hledgerExecutablesInPath                                  -- ["hledger-check","hledger-check-dates","hledger-check-dates.hs","hledger-check.hs","hledger-check.py"]
-  let as2 = map stripPrognamePrefix as1                            -- ["check","check-dates","check-dates.hs","check.hs","check.py"]
-  let as3 = sortBy (comparing takeBaseName) as2                    -- ["check","check.hs","check.py","check-dates","check-dates.hs"]
-  let as4 = groupBy (\a b -> takeBaseName a == takeBaseName b) as3 -- [["check","check.hs","check.py"],["check-dates","check-dates.hs"]]
-  let as5 = concatMap dropRedundantSourceVersion as4               -- ["check","check.hs","check.py","check-dates"]
-  return as5
+  as1 <- hledgerExecutablesInPath                     -- ["hledger-check","hledger-check-dates","hledger-check-dates.hs","hledger-check.hs","hledger-check.py"]
+  let as2 = map stripPrognamePrefix as1               -- ["check","check-dates","check-dates.hs","check.hs","check.py"]
+  let as3 = groupSortOn takeBaseName as2              -- [["check","check.hs","check.py"],["check-dates","check-dates.hs"]]
+  let as4 = concatMap dropRedundantSourceVersion as3  -- ["check","check.hs","check.py","check-dates"]
+  return as4
 
 stripPrognamePrefix = drop (length progname + 1)
 
@@ -768,3 +777,21 @@ getDirectoryContentsSafe d =
 --     d <- getCurrentDay
 --     putStrLn $ "search query: " ++ (show $ queryFromOpts d $ reportopts_ opts)
 
+-- ** Lenses
+
+makeHledgerClassyLenses ''CliOpts
+
+instance HasInputOpts CliOpts where
+    inputOpts = inputopts
+
+instance HasBalancingOpts CliOpts where
+    balancingOpts = inputOpts.balancingOpts
+
+instance HasReportSpec CliOpts where
+    reportSpec = reportspec
+
+instance HasReportOptsNoUpdate CliOpts where
+    reportOptsNoUpdate = reportSpec.reportOptsNoUpdate
+
+instance HasReportOpts CliOpts where
+    reportOpts = reportSpec.reportOpts
