@@ -5,7 +5,6 @@ New built-in commands should be added in four places below:
 the export list, the import list, builtinCommands, commandsList.
 -}
 
-{-# LANGUAGE CPP #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE QuasiQuotes #-}
 {-# LANGUAGE TemplateHaskell #-}
@@ -25,8 +24,6 @@ module Hledger.Cli.Commands (
   ,module Hledger.Cli.Commands.Balancesheet
   ,module Hledger.Cli.Commands.Balancesheetequity
   ,module Hledger.Cli.Commands.Cashflow
-  ,module Hledger.Cli.Commands.Checkdates
-  ,module Hledger.Cli.Commands.Checkdupes
   ,module Hledger.Cli.Commands.Close
   ,module Hledger.Cli.Commands.Codes
   ,module Hledger.Cli.Commands.Commodities
@@ -49,11 +46,7 @@ module Hledger.Cli.Commands (
 where
 
 import Data.Char (isSpace)
-import Data.Default
 import Data.List
-#if !(MIN_VERSION_base(4,11,0))
-import Data.Monoid ((<>))
-#endif
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Time.Calendar
@@ -72,8 +65,7 @@ import Hledger.Cli.Commands.Balance
 import Hledger.Cli.Commands.Balancesheet
 import Hledger.Cli.Commands.Balancesheetequity
 import Hledger.Cli.Commands.Cashflow
-import Hledger.Cli.Commands.Checkdates
-import Hledger.Cli.Commands.Checkdupes
+import Hledger.Cli.Commands.Check
 import Hledger.Cli.Commands.Close
 import Hledger.Cli.Commands.Codes
 import Hledger.Cli.Commands.Commodities
@@ -109,8 +101,7 @@ builtinCommands = [
   ,(balancesheetequitymode , balancesheetequity)
   ,(balancesheetmode       , balancesheet)
   ,(cashflowmode           , cashflow)
-  ,(checkdatesmode         , checkdates)
-  ,(checkdupesmode         , checkdupes)
+  ,(checkmode              , check)
   ,(closemode              , close)
   ,(codesmode              , codes)
   ,(commoditiesmode        , commodities)
@@ -154,6 +145,7 @@ builtinCommands = [
 -- 
 commandsList :: String -> [String] -> [String]
 commandsList progversion othercmds = [
+  -- keep synced with hledger.m4.md -> Commands, commands.m4 -->
    "-------------------------------------------------------------------------------"
   ,progversion
   ,"Usage: hledger COMMAND [OPTIONS] [-- ADDONCMDOPTIONS]"
@@ -166,13 +158,14 @@ commandsList progversion othercmds = [
   ,""
   ,"Data management:"
   ,"+autosync                 download/deduplicate/convert OFX data"
-  ,"+check                    check more powerful balance assertions"
-  ," check-dates              check transactions are ordered by date"
-  ," check-dupes              check for accounts with the same leaf name"
-  ," close (equity)           generate balance-resetting transactions"
+  ," check                    check for various kinds of issue in the data"
+  ,"+check-fancyassertions    check more powerful balance assertions"
+  ,"+check-tagfiles           check file paths in tag values exist"
+  ," close                    generate balance-resetting transactions"
   ," diff                     compare account transactions in two journal files"
   ,"+interest                 generate interest transactions"
-  ," rewrite                  generate automated postings/diffs (old, use --auto)"
+  ," rewrite                  generate extra postings, similar to print --auto"
+  ,"+stockquotes              download market prices from AlphaVantage"
   ,""
   ,"Financial reports:"
   ," aregister (areg)         show transactions in a particular account"
@@ -180,13 +173,12 @@ commandsList progversion othercmds = [
   ," balancesheetequity (bse) show assets, liabilities and equity"
   ," cashflow (cf)            show changes in liquid assets"
   ," incomestatement (is)     show revenues and expenses"
-  ,"+irr                      calculate internal rate of return (old, use roi)"
   ," roi                      show return on investments"
   ,""
   ,"Low-level reports:"
-  ," accounts (a)             show account names"
+  ," accounts                 show account names"
   ," activity                 show postings-per-interval bar charts"
-  ," balance (b, bal)         show balance changes/end balances/budgets in accounts"
+  ," balance (bal)            show balance changes/end balances/budgets in accounts"
   ," codes                    show transaction codes"
   ," commodities              show commodity/currency symbols"
   ," descriptions             show unique transaction descriptions"
@@ -194,33 +186,31 @@ commandsList progversion othercmds = [
   ," notes                    show unique note segments of transaction descriptions"
   ," payees                   show unique payee segments of transaction descriptions"
   ," prices                   show market price records"
-  ," print (p, txns)          show transactions (journal entries)"
+  ," print                    show transactions (journal entries)"
   ," print-unique             show only transactions with unique descriptions"
-  ," register (r, reg)        show postings in one or more accounts & running total"
+  ," register (reg)           show postings in one or more accounts & running total"
   ," register-match           show a recent posting that best matches a description"
   ," stats                    show journal statistics"
   ," tags                     show tag names"
   ," test                     run self tests"
   ,""
   ,"Alternate user interfaces:"
-  ,"+ui                       run curses ui"
-  ,"+web                      run web ui"
-  ,"+api                      run http api server"
+  ,"+ui                       run terminal UI"
+  ,"+web                      run web UI"
   ,""
   ,"Other:"
   ] ++
   othercmds
   ++
-  ["Help:"
+  [""
+  ,"Help:"
   ," (no arguments)           show this commands list"
   ," -h                       show general flags"
   ," COMMAND -h               show flags & docs for COMMAND"
   ," help [MANUAL]            show hledger manuals in various formats"
   ,""
   ]
--- commands                 show brief commands list
 -- edit                     open a text editor on some part of the journal
--- aregister (ar, areg)     show transactions in a single account
 
 
 -- | All names and aliases of builtin commands.
@@ -238,15 +228,15 @@ commandsFromCommandsList s =
   [w | c:l <- s, c `elem` [' ','+'], let w:_ = words l]
 
 knownCommands :: [String]
-knownCommands = sort . commandsFromCommandsList $ commandsList prognameandversion []
+knownCommands = sort . commandsFromCommandsList . drop 1 $ commandsList progname []  -- progname will not be seen
 
 -- | Print the commands list, modifying the template above based on
 -- the currently available addons. Missing addons will be removed, and
 -- extra addons will be added under Misc.
-printCommandsList :: [String] -> IO ()
-printCommandsList addonsFound =
+printCommandsList :: String -> [String] -> IO ()
+printCommandsList progversion addonsFound =
     putStr . unlines . concatMap adjustline $
-    commandsList prognameandversion (map ('+':) unknownCommandsFound)
+    commandsList progversion (map ('+':) unknownCommandsFound)
   where
     commandsFound = map (' ':) builtinCommandNames ++ map ('+':) addonsFound
     unknownCommandsFound = addonsFound \\ knownCommands
@@ -276,34 +266,34 @@ testmode = hledgerCommandMode
 -- not be used (and would raise an error).
 --
 testcmd :: CliOpts -> Journal -> IO ()
-testcmd opts _undefined = do 
-  withArgs (words' $ query_ $ reportopts_ opts) $
-    Test.Tasty.defaultMain $ tests "hledger" [
+testcmd opts _undefined = do
+  withArgs (listofstringopt "args" $ rawopts_ opts) $
+    Test.Tasty.defaultMain $ testGroup "hledger" [
        tests_Hledger
       ,tests_Hledger_Cli
       ]
 
 -- All unit tests for Hledger.Cli, defined here rather than
 -- Hledger.Cli so testcmd can use them.
-tests_Hledger_Cli = tests "Hledger.Cli" [
+tests_Hledger_Cli = testGroup "Hledger.Cli" [
    tests_Cli_Utils
   ,tests_Commands
   ]
 
-tests_Commands = tests "Commands" [
+tests_Commands = testGroup "Commands" [
    tests_Balance
   ,tests_Register
   ,tests_Aregister
 
   -- some more tests easiest to define here:
 
-  ,tests "apply account directive" [
-     test "works" $ do
+  ,testGroup "apply account directive" [
+     testCase "works" $ do
         let
           ignoresourcepos j = j{jtxns=map (\t -> t{tsourcepos=nullsourcepos}) (jtxns j)}
           sameParse str1 str2 = do
-            j1 <- readJournal def Nothing str1 >>= either error' (return . ignoresourcepos)  -- PARTIAL:
-            j2 <- readJournal def Nothing str2 >>= either error' (return . ignoresourcepos)
+            j1 <- readJournal definputopts Nothing str1 >>= either error' (return . ignoresourcepos)  -- PARTIAL:
+            j2 <- readJournal definputopts Nothing str2 >>= either error' (return . ignoresourcepos)
             j1 @?= j2{jlastreadtime=jlastreadtime j1, jfiles=jfiles j1} --, jparsestate=jparsestate j1}
         sameParse
            ("2008/12/07 One\n  alpha  $-1\n  beta  $1\n" <>
@@ -319,43 +309,43 @@ tests_Commands = tests "Commands" [
             "2008/12/07 Five\n  foo  $-5\n  bar  $5\n"
            )
 
-    ,test "preserves \"virtual\" posting type" $ do
-      j <- readJournal def Nothing "apply account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n" >>= either error' return  -- PARTIAL:
+    ,testCase "preserves \"virtual\" posting type" $ do
+      j <- readJournal definputopts Nothing "apply account test\n2008/12/07 One\n  (from)  $-1\n  (to)  $1\n" >>= either error' return  -- PARTIAL:
       let p = head $ tpostings $ head $ jtxns j
       paccount p @?= "test:from"
       ptype p @?= VirtualPosting
     ]
 
-  ,test "alias directive" $ do
-    j <- readJournal def Nothing "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n" >>= either error' return  -- PARTIAL:
+  ,testCase "alias directive" $ do
+    j <- readJournal definputopts Nothing "!alias expenses = equity:draw:personal\n1/1\n (expenses:food)  1\n" >>= either error' return  -- PARTIAL:
     let p = head $ tpostings $ head $ jtxns j
     paccount p @?= "equity:draw:personal:food"
 
-  ,test "Y default year directive" $ do
-    j <- readJournal def Nothing defaultyear_journal_txt >>= either error' return  -- PARTIAL:
+  ,testCase "Y default year directive" $ do
+    j <- readJournal definputopts Nothing defaultyear_journal_txt >>= either error' return  -- PARTIAL:
     tdate (head $ jtxns j) @?= fromGregorian 2009 1 1
 
-  ,test "ledgerAccountNames" $
+  ,testCase "ledgerAccountNames" $
     (ledgerAccountNames ledger7)
     @?=
     ["assets","assets:cash","assets:checking","assets:saving","equity","equity:opening balances",
      "expenses","expenses:food","expenses:food:dining","expenses:phone","expenses:vacation",
      "liabilities","liabilities:credit cards","liabilities:credit cards:discover"]
 
-  -- ,test "journalCanonicaliseAmounts" ~:
+  -- ,testCase "journalCanonicaliseAmounts" ~:
   --  "use the greatest precision" ~:
   --   (map asprecision $ journalAmountAndPriceCommodities $ journalCanonicaliseAmounts $ journalWithAmounts ["1","2.00"]) @?= [2,2]
 
   -- don't know what this should do
-  -- ,test "elideAccountName" ~: do
+  -- ,testCase "elideAccountName" ~: do
   --    (elideAccountName 50 "aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa"
   --     @?= "aa:aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa")
   --    (elideAccountName 20 "aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa:aaaaaaaaaaaaaaaaaaaa"
   --     @?= "aa:aa:aaaaaaaaaaaaaa")
 
-  ,test "show dollars" $ showAmount (usd 1) @?= "$1.00"
+  ,testCase "show dollars" $ showAmount (usd 1) @?= "$1.00"
 
-  ,test "show hours" $ showAmount (hrs 1) @?= "1.00h"
+  ,testCase "show hours" $ showAmount (hrs 1) @?= "1.00h"
 
   ]
 
@@ -502,7 +492,7 @@ defaultyear_journal_txt = T.unlines
 --  ]
 
 -- journal4_str = unlines
---  ["!include \"somefile\""
+--  ["include \"somefile\""
 --  ,"2007/01/27 * joes diner"
 --  ,"  expenses:food:dining                    $10.00"
 --  ,"  assets:checking                        $-47.18"
