@@ -14,16 +14,12 @@ The @accounts@ command lists account names:
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE TemplateHaskell #-}
-{-# LANGUAGE CPP #-}
 
 module Hledger.Cli.Commands.Accounts (
   accountsmode
  ,accounts
 ) where
 
-#if !(MIN_VERSION_base(4,11,0))
-import Data.Monoid
-#endif
 import Data.List
 import qualified Data.Text as T
 import qualified Data.Text.IO as T
@@ -31,6 +27,7 @@ import System.Console.CmdArgs.Explicit as C
 
 import Hledger
 import Hledger.Cli.CliOptions
+import Control.Monad (forM_)
 
 
 -- | Command line options for this command.
@@ -38,6 +35,7 @@ accountsmode = hledgerCommandMode
   $(embedFileRelative "Hledger/Cli/Commands/Accounts.txt")
   ([flagNone ["declared"] (setboolopt "declared") "show account names declared with account directives"
   ,flagNone ["used"] (setboolopt "used") "show account names referenced by transactions"
+  ,flagNone ["types"] (setboolopt "types") "also show accounts' types, when known"
   ]
   ++ flattreeflags False ++
   [flagReq  ["drop"] (\s opts -> Right $ setopt "drop" s opts) "N" "flat mode: omit N leading account name parts"
@@ -48,20 +46,22 @@ accountsmode = hledgerCommandMode
 
 -- | The accounts command.
 accounts :: CliOpts -> Journal -> IO ()
-accounts CliOpts{rawopts_=rawopts, reportopts_=ropts} j = do
+accounts CliOpts{rawopts_=rawopts, reportspec_=ReportSpec{_rsQuery=query,_rsReportOpts=ropts}} j = do
 
   -- 1. identify the accounts we'll show
-  d <- getCurrentDay
   let tree     = tree_ ropts
       declared = boolopt "declared" rawopts
       used     = boolopt "used"     rawopts
-      q        = queryFromOpts d ropts
+      types    = boolopt "types"    rawopts
       -- a depth limit will clip and exclude account names later, but we don't want to exclude accounts at this stage
-      nodepthq = dbg1 "nodepthq" $ filterQuery (not . queryIsDepth) q
+      nodepthq = dbg1 "nodepthq" $ filterQuery (not . queryIsDepth) query
       -- just the acct: part of the query will be reapplied later, after clipping
-      acctq    = dbg1 "acctq" $ filterQuery queryIsAcct q
-      depth    = dbg1 "depth" $ queryDepth $ filterQuery queryIsDepth q
-      matcheddeclaredaccts = dbg1 "matcheddeclaredaccts" $ filter (matchesAccount nodepthq) $ map fst $ jdeclaredaccounts j
+      acctq    = dbg1 "acctq" $ filterQuery queryIsAcct query
+      depth    = dbg1 "depth" $ queryDepth $ filterQuery queryIsDepth query
+      matcheddeclaredaccts =
+        dbg1 "matcheddeclaredaccts" $
+        filter (matchesAccountExtra (journalAccountType j) (journalInheritedAccountTags j) nodepthq)
+          $ map fst $ jdeclaredaccounts j
       matchedusedaccts     = dbg5 "matchedusedaccts" $ map paccount $ journalPostings $ filterJournalPostings nodepthq j
       accts                = dbg5 "accts to show" $ -- no need to nub/sort, accountTree will
         if | declared     && not used -> matcheddeclaredaccts
@@ -80,12 +80,20 @@ accounts CliOpts{rawopts_=rawopts, reportopts_=ropts} j = do
         map (clipAccountName depth) $  -- clip at depth if specified
         sortedaccts
 
-  -- 4. print what remains as a list or tree, maybe applying --drop in the former case
-  mapM_ (T.putStrLn . render) clippedaccts
-    where
-      render a = case accountlistmode_ ropts of
-          ALTree -> T.replicate indent " " <> accountLeafName droppedName
-          ALFlat -> droppedName
-        where
-          indent = 2 * (max 0 (accountNameLevel a - drop_ ropts) - 1)
-          droppedName = accountNameDrop (drop_ ropts) a
+  -- 4. print what remains as a list or tree, maybe applying --drop in the former case.
+  -- With --types, also show the account type.
+  let
+    -- some contortions here to show types nicely aligned
+    showName a = case accountlistmode_ ropts of
+      ALTree -> indent <> accountLeafName droppedName
+      ALFlat -> droppedName
+      where
+        indent      = T.replicate (2 * (max 0 (accountNameLevel a - drop_ ropts) - 1)) " "
+        droppedName = accountNameDrop (drop_ ropts) a
+    showType a 
+      | types     = spacer <> "    ; type: " <> maybe "" (T.pack . show) (journalAccountType j a)
+      | otherwise = ""
+      where
+        spacer = T.replicate (maxwidth - T.length (showName a)) " "
+    maxwidth = maximum $ map (T.length . showName) clippedaccts
+  forM_ clippedaccts $ \a -> T.putStrLn $ showName a <> showType a
