@@ -11,12 +11,9 @@ module Hledger.Cli.Commands.Add (
   ,add
   ,appendToJournalFileOrStdout
   ,journalAddTransaction
-  ,transactionsSimilarTo
 )
 where
 
-import Prelude ()
-import "base-compat-batteries" Prelude.Compat hiding (fail)
 import Control.Exception as E
 import Control.Monad (when)
 import Control.Monad.Trans.Class
@@ -25,18 +22,21 @@ import Control.Monad.Trans (liftIO)
 import Data.Char (toUpper, toLower)
 import Data.Either (isRight)
 import Data.Functor.Identity (Identity(..))
-import "base-compat-batteries" Data.List.Compat
-import qualified Data.Set as S
-import Data.Maybe
+import Data.List (isPrefixOf)
+import Data.Maybe (fromJust, fromMaybe, isJust)
 import Data.Text (Text)
 import qualified Data.Text as T
+import qualified Data.Text.IO as T
+import qualified Data.Text.Lazy as TL
+import qualified Data.Text.Lazy.IO as TL
 import Data.Time.Calendar (Day)
 import Data.Time.Format (formatTime, defaultTimeLocale, iso8601DateFormat)
+import Lens.Micro ((^.))
 import Safe (headDef, headMay, atMay)
-import System.Console.CmdArgs.Explicit
+import System.Console.CmdArgs.Explicit (flagNone)
 import System.Console.Haskeline (runInputT, defaultSettings, setComplete)
-import System.Console.Haskeline.Completion
-import System.Console.Wizard
+import System.Console.Haskeline.Completion (CompletionFunc, completeWord, isFinished, noCompletion, simpleCompletion)
+import System.Console.Wizard (Wizard, defaultTo, line, output, retryMsg, linePrewritten, nonEmpty, parser, run)
 import System.Console.Wizard.Haskeline
 import System.IO ( stderr, hPutStr, hPutStrLn )
 import Text.Megaparsec
@@ -46,6 +46,7 @@ import Text.Printf
 import Hledger
 import Hledger.Cli.CliOptions
 import Hledger.Cli.Commands.Register (postingsReportAsText)
+import Hledger.Cli.Utils (journalSimilarTransaction)
 
 
 addmode = hledgerCommandMode
@@ -89,10 +90,10 @@ add :: CliOpts -> Journal -> IO ()
 add opts j
     | journalFilePath j == "-" = return ()
     | otherwise = do
-        hPrintf stderr "Adding transactions to journal file %s\n" (journalFilePath j)
+        hPutStrLn stderr $ "Adding transactions to journal file " <> journalFilePath j
         showHelp
-        today <- getCurrentDay
-        let es = defEntryState{esOpts=opts
+        let today = opts^.rsDay
+            es = defEntryState{esOpts=opts
                               ,esArgs=listofstringopt "args" $ rawopts_ opts
                               ,esToday=today
                               ,esDefDate=today
@@ -123,16 +124,16 @@ getAndAddTransactions es@EntryState{..} = (do
     Nothing -> error "Could not interpret the input, restarting"  -- caught below causing a restart, I believe  -- PARTIAL:
     Just t -> do
       j <- if debug_ esOpts > 0
-           then do hPrintf stderr "Skipping journal add due to debug mode.\n"
+           then do hPutStrLn stderr "Skipping journal add due to debug mode."
                    return esJournal
            else do j' <- journalAddTransaction esJournal esOpts t
-                   hPrintf stderr "Saved.\n"
+                   hPutStrLn stderr "Saved."
                    return j'
-      hPrintf stderr "Starting the next transaction (. or ctrl-D/ctrl-C to quit)\n"
+      hPutStrLn stderr "Starting the next transaction (. or ctrl-D/ctrl-C to quit)"
       getAndAddTransactions es{esJournal=j, esDefDate=tdate t}
   )
   `E.catch` (\(_::RestartTransactionException) ->
-                 hPrintf stderr "Restarting this transaction.\n" >> getAndAddTransactions es)
+                 hPutStrLn stderr "Restarting this transaction." >> getAndAddTransactions es)
 
 data TxnParams = TxnParams
   { txnDate :: Day
@@ -164,7 +165,8 @@ confirmedTransactionWizard prevInput es@EntryState{..} stack@(currentStage : _)
             { esArgs = drop 1 esArgs
             , esDefDate = date
             }
-          dateAndCodeString = formatTime defaultTimeLocale yyyymmddFormat date ++ (if T.null code then "" else " (" ++ T.unpack code ++ ")")
+          dateAndCodeString = formatTime defaultTimeLocale yyyymmddFormat date
+                            ++ T.unpack (if T.null code then "" else " (" <> code <> ")")
           yyyymmddFormat = iso8601DateFormat Nothing
       confirmedTransactionWizard prevInput{prevDateAndCode=Just dateAndCodeString} es' (EnterDescAndComment (date, code) : stack)
     Nothing ->
@@ -172,7 +174,7 @@ confirmedTransactionWizard prevInput es@EntryState{..} stack@(currentStage : _)
 
   EnterDescAndComment (date, code) -> descriptionAndCommentWizard prevInput es >>= \case
     Just (desc, comment) -> do
-      let mbaset = similarTransaction es desc
+      let mbaset = journalSimilarTransaction esOpts esJournal desc
           es' = es
             { esArgs = drop 1 esArgs
             , esPostings = []
@@ -180,7 +182,9 @@ confirmedTransactionWizard prevInput es@EntryState{..} stack@(currentStage : _)
             }
           descAndCommentString = T.unpack $ desc <> (if T.null comment then "" else "  ; " <> comment)
           prevInput' = prevInput{prevDescAndCmnt=Just descAndCommentString}
-      when (isJust mbaset) $ liftIO $ hPrintf stderr "Using this similar transaction for defaults:\n%s" (showTransaction $ fromJust mbaset)
+      when (isJust mbaset) . liftIO $ do
+          hPutStrLn stderr "Using this similar transaction for defaults:"
+          T.hPutStr stderr $ showTransaction (fromJust mbaset)
       confirmedTransactionWizard prevInput' es' ((EnterNewPosting TxnParams{txnDate=date, txnCode=code, txnDesc=desc, txnCmnt=comment} Nothing) : stack)
     Nothing ->
       confirmedTransactionWizard prevInput es (drop 1 stack)
@@ -198,7 +202,7 @@ confirmedTransactionWizard prevInput es@EntryState{..} stack@(currentStage : _)
                              ,tcomment=txnCmnt
                              ,tpostings=esPostings
                              }
-      case balanceTransaction Nothing t of -- imprecise balancing (?)
+      case balanceTransaction defbalancingopts t of -- imprecise balancing (?)
         Right t' ->
           confirmedTransactionWizard prevInput es (EndStage t' : stack)
         Left err -> do
@@ -228,18 +232,18 @@ confirmedTransactionWizard prevInput es@EntryState{..} stack@(currentStage : _)
   EnterAmountAndComment txnParams account -> amountAndCommentWizard prevInput es >>= \case
     Just (amount, comment) -> do
       let posting = nullposting{paccount=T.pack $ stripbrackets account
-                               ,pamount=Mixed [amount]
+                               ,pamount=mixedAmount amount
                                ,pcomment=comment
                                ,ptype=accountNamePostingType $ T.pack account
                                }
-          amountAndCommentString = showAmount amount ++ (if T.null comment then "" else "  ;" ++ T.unpack comment)
+          amountAndCommentString = showAmount amount ++ T.unpack (if T.null comment then "" else "  ;" <> comment)
           prevAmountAndCmnt' = replaceNthOrAppend (length esPostings) amountAndCommentString (prevAmountAndCmnt prevInput)
           es' = es{esPostings=esPostings++[posting], esArgs=drop 2 esArgs}
       confirmedTransactionWizard prevInput{prevAmountAndCmnt=prevAmountAndCmnt'} es' (EnterNewPosting txnParams (Just posting) : stack)
     Nothing -> confirmedTransactionWizard prevInput es (drop 1 stack)
 
   EndStage t -> do
-    output $ showTransaction t
+    output . T.unpack $ showTransaction t
     y <- let def = "y" in
          retryMsg "Please enter y or n." $
           parser ((fmap (\c -> if c == '<' then Nothing else Just c)) . headMay . map toLower . strip) $
@@ -252,17 +256,8 @@ confirmedTransactionWizard prevInput es@EntryState{..} stack@(currentStage : _)
   where
     replaceNthOrAppend n newElem xs = take n xs ++ [newElem] ++ drop (n + 1) xs
 
--- Identify the closest recent match for this description in past transactions.
-similarTransaction :: EntryState -> Text -> Maybe Transaction
-similarTransaction EntryState{..} desc =
-  let q = queryFromOptsOnly esToday $ reportopts_ esOpts
-      historymatches = transactionsSimilarTo esJournal q desc
-      bestmatch | null historymatches = Nothing
-                | otherwise           = Just $ snd $ head historymatches
-  in bestmatch
-
 dateAndCodeWizard PrevInput{..} EntryState{..} = do
-  let def = headDef (showDate esDefDate) esArgs
+  let def = headDef (T.unpack $ showDate esDefDate) esArgs
   retryMsg "A valid hledger smart date is required. Eg: 2014/2/14, 14, yesterday." $
    parser (parseSmartDateAndCode esToday) $
    withCompletion (dateCompleter def) $
@@ -296,14 +291,14 @@ descriptionAndCommentWizard PrevInput{..} EntryState{..} = do
       return $ Just (desc, comment)
 
 postingsBalanced :: [Posting] -> Bool
-postingsBalanced ps = isRight $ balanceTransaction Nothing nulltransaction{tpostings=ps}
+postingsBalanced ps = isRight $ balanceTransaction defbalancingopts nulltransaction{tpostings=ps}
 
 accountWizard PrevInput{..} EntryState{..} = do
   let pnum = length esPostings + 1
       historicalp = fmap ((!! (pnum - 1)) . (++ (repeat nullposting)) . tpostings) esSimilarTransaction
       historicalacct = case historicalp of Just p  -> showAccountName Nothing (ptype p) (paccount p)
                                            Nothing -> ""
-      def = headDef historicalacct esArgs
+      def = headDef (T.unpack historicalacct) esArgs
       endmsg | canfinish && null def = " (or . or enter to finish this transaction)"
              | canfinish             = " (or . to finish this transaction)"
              | otherwise             = ""
@@ -333,17 +328,21 @@ amountAndCommentWizard PrevInput{..} EntryState{..} = do
       (mhistoricalp,followedhistoricalsofar) =
           case esSimilarTransaction of
             Nothing                        -> (Nothing,False)
-            Just Transaction{tpostings=ps} -> (if length ps >= pnum then Just (ps !! (pnum-1)) else Nothing
-                                              ,all (\(a,b) -> pamount a == pamount b) $ zip esPostings ps)
-      def = case (esArgs, mhistoricalp, followedhistoricalsofar) of
-              (d:_,_,_)                                             -> d
-              (_,Just hp,True)                                      -> showamt $ pamount hp
-              _  | pnum > 1 && not (mixedAmountLooksZero balancingamt) -> showamt balancingamtfirstcommodity
-              _                                                     -> ""
+            Just Transaction{tpostings=ps} ->
+              ( if length ps >= pnum then Just (ps !! (pnum-1)) else Nothing
+              , all sameamount $ zip esPostings ps
+              )
+              where
+                sameamount (p1,p2) = mixedAmountUnstyled (pamount p1) == mixedAmountUnstyled (pamount p2)
+      def | (d:_) <- esArgs                                     = d
+          | Just hp <- mhistoricalp, followedhistoricalsofar    = showamt $ pamount hp
+          | pnum > 1 && not (mixedAmountLooksZero balancingamt) = showamt balancingamtfirstcommodity
+          | otherwise                                           = ""
   retryMsg "A valid hledger amount is required. Eg: 1, $2, 3 EUR, \"4 red apples\"." $
    parser parseAmountAndComment $
    withCompletion (amountCompleter def) $
-   defaultTo' def $ nonEmpty $
+   defaultTo' def $
+   nonEmpty $
    linePrewritten (green $ printf "Amount  %d%s: " pnum (showDefault def)) (fromMaybe "" $ prevAmountAndCmnt `atMay` length esPostings) ""
     where
       parseAmountAndComment s = if s == "<" then return Nothing else either (const Nothing) (return . Just) $
@@ -359,10 +358,9 @@ amountAndCommentWizard PrevInput{..} EntryState{..} = do
         c <- T.pack <$> fromMaybe "" `fmap` optional (char ';' >> many anySingle)
         -- eof
         return (a,c)
-      balancingamt = negate $ sum $ map pamount realps where realps = filter isReal esPostings
-      balancingamtfirstcommodity = Mixed $ take 1 $ amounts balancingamt
-      showamt =
-        showMixedAmountWithPrecision
+      balancingamt = maNegate . sumPostings $ filter isReal esPostings
+      balancingamtfirstcommodity = mixed . take 1 $ amounts balancingamt
+      showamt = wbUnpack . showMixedAmountB noColour . mixedAmountSetPrecision
                   -- what should this be ?
                   -- 1 maxprecision (show all decimal places or none) ?
                   -- 2 maxprecisionwithpoint (show all decimal places or .0 - avoids some but not all confusion with thousands separators) ?
@@ -437,7 +435,7 @@ journalAddTransaction j@Journal{jtxns=ts} opts t = do
     -- unelided shows all amounts explicitly, in case there's a price, cf #283
   when (debug_ opts > 0) $ do
     putStrLn $ printf "\nAdded transaction to %s:" f
-    putStrLn =<< registerFromString (showTransaction t)
+    TL.putStrLn =<< registerFromString (showTransaction t)
   return j{jtxns=ts++[t]}
 
 -- | Append a string, typically one or more transactions, to a journal
@@ -448,72 +446,26 @@ journalAddTransaction j@Journal{jtxns=ts} opts t = do
 -- even if the file uses dos line endings (\r\n), which could leave
 -- mixed line endings in the file. See also writeFileWithBackupIfChanged.
 --
-appendToJournalFileOrStdout :: FilePath -> String -> IO ()
+appendToJournalFileOrStdout :: FilePath -> Text -> IO ()
 appendToJournalFileOrStdout f s
-  | f == "-"  = putStr s'
-  | otherwise = appendFile f s'
-  where s' = "\n" ++ ensureOneNewlineTerminated s
+  | f == "-"  = T.putStr s'
+  | otherwise = appendFile f $ T.unpack s'
+  where s' = "\n" <> ensureOneNewlineTerminated s
 
 -- | Replace a string's 0 or more terminating newlines with exactly one.
-ensureOneNewlineTerminated :: String -> String
-ensureOneNewlineTerminated = (++"\n") . reverse . dropWhile (=='\n') . reverse
+ensureOneNewlineTerminated :: Text -> Text
+ensureOneNewlineTerminated = (<>"\n") . T.dropWhileEnd (=='\n')
 
 -- | Convert a string of journal data into a register report.
-registerFromString :: String -> IO String
+registerFromString :: T.Text -> IO TL.Text
 registerFromString s = do
-  d <- getCurrentDay
-  j <- readJournal' $ T.pack s
-  return $ postingsReportAsText opts $ postingsReport ropts (queryFromOpts d ropts) j
+  j <- readJournal' s
+  return . postingsReportAsText opts $ postingsReport rspec j
       where
         ropts = defreportopts{empty_=True}
-        opts = defcliopts{reportopts_=ropts}
+        rspec = defreportspec{_rsReportOpts=ropts}
+        opts = defcliopts{reportspec_=rspec}
 
 capitalize :: String -> String
 capitalize "" = ""
 capitalize (c:cs) = toUpper c : cs
-
--- | Find the most similar and recent transactions matching the given
--- transaction description and report query.  Transactions are listed
--- with their "relevancy" score, most relevant first.
-transactionsSimilarTo :: Journal -> Query -> Text -> [(Double,Transaction)]
-transactionsSimilarTo j q desc =
-    sortBy compareRelevanceAndRecency
-               $ filter ((> threshold).fst)
-               [(compareDescriptions desc $ tdescription t, t) | t <- ts]
-    where
-      compareRelevanceAndRecency (n1,t1) (n2,t2) = compare (n2,tdate t2) (n1,tdate t1)
-      ts = filter (q `matchesTransaction`) $ jtxns j
-      threshold = 0
-
--- | Return a similarity measure, from 0 to 1, for two transaction
--- descriptions.  This is like compareStrings, but first strips out
--- any numbers, to improve accuracy eg when there are bank transaction
--- ids from imported data.
-compareDescriptions :: Text -> Text -> Double
-compareDescriptions s t = compareStrings s' t'
-    where s' = simplify $ T.unpack s
-          t' = simplify $ T.unpack t
-          simplify = filter (not . (`elem` ("0123456789" :: String)))
-
--- | Return a similarity measure, from 0 to 1, for two strings.  This
--- was based on Simon White's string similarity algorithm
--- (http://www.catalysoft.com/articles/StrikeAMatch.html), later found
--- to be https://en.wikipedia.org/wiki/S%C3%B8rensen%E2%80%93Dice_coefficient,
--- modified to handle short strings better.
--- Todo: check out http://nlp.fi.muni.cz/raslan/2008/raslan08.pdf#page=14 .
-compareStrings :: String -> String -> Double
-compareStrings "" "" = 1
-compareStrings [_] "" = 0
-compareStrings "" [_] = 0
-compareStrings [a] [b] = if toUpper a == toUpper b then 1 else 0
-compareStrings s1 s2 = 2 * commonpairs / totalpairs
-    where
-      pairs1      = S.fromList $ wordLetterPairs $ uppercase s1
-      pairs2      = S.fromList $ wordLetterPairs $ uppercase s2
-      commonpairs = fromIntegral $ S.size $ S.intersection pairs1 pairs2
-      totalpairs  = fromIntegral $ S.size pairs1 + S.size pairs2
-
-wordLetterPairs = concatMap letterPairs . words
-
-letterPairs (a:b:rest) = [a,b] : letterPairs (b:rest)
-letterPairs _ = []
