@@ -32,11 +32,8 @@ codesmode = hledgerCommandMode
 
 -- | The codes command.
 codes :: CliOpts -> Journal -> IO ()
-codes CliOpts{reportopts_=ropts@ReportOpts{empty_}} j = do
-  d <- getCurrentDay
-  let q  = queryFromOpts d ropts
-      ts = entriesReport ropts q j
-      codes = (if empty_ then id else filter (not . T.null)) $
+codes CliOpts{reportspec_=rspec} j = do
+  let ts = entriesReport rspec j
+      codes = (if empty_ (_rsReportOpts rspec) then id else filter (not . T.null)) $
               map tcode ts
-
   mapM_ T.putStrLn codes
