@@ -31,10 +31,8 @@ descriptionsmode = hledgerCommandMode
 
 -- | The descriptions command.
 descriptions :: CliOpts -> Journal -> IO ()
-descriptions CliOpts{reportopts_=ropts} j = do
-  d <- getCurrentDay
-  let q  = queryFromOpts d ropts
-      ts = entriesReport ropts q j
+descriptions CliOpts{reportspec_=rspec} j = do
+  let ts = entriesReport rspec j
       descriptions = nubSort $ map tdescription ts
 
   mapM_ T.putStrLn descriptions
