@@ -1,4 +1,5 @@
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE TemplateHaskell   #-}
 
 module Hledger.Cli.Commands.Prices (
   pricesmode
@@ -10,6 +11,7 @@ import qualified Data.Map as M
 import Data.Maybe
 import Data.List
 import qualified Data.Text as T
+import qualified Data.Text.IO as T
 import Data.Time
 import Hledger
 import Hledger.Cli.CliOptions
@@ -17,24 +19,35 @@ import System.Console.CmdArgs.Explicit
 
 pricesmode = hledgerCommandMode
   $(embedFileRelative "Hledger/Cli/Commands/Prices.txt")
-  [flagNone ["costs"] (setboolopt "costs") "print transaction prices from postings"
-  ,flagNone ["inverted-costs"] (setboolopt "inverted-costs") "print transaction inverted prices from postings also"]
+  [flagNone ["infer-reverse-prices"] (setboolopt "infer-reverse-prices") "also show prices obtained by inverting transaction prices"
+  ]
   [generalflagsgroup1]
-  hiddenflags
+  (hiddenflags ++
+  [flagNone ["costs"]          (setboolopt "infer-market-prices") "deprecated, use --infer-market-prices instead"
+  ,flagNone ["inverted-costs"] (setboolopt "infer-reverse-prices")      "deprecated, use --infer-reverse-prices instead"
+  ])
   ([], Just $ argsFlag "[QUERY]")
 
 -- XXX the original hledger-prices script always ignored assertions
 prices opts j = do
-  d <- getCurrentDay
   let
     styles     = journalCommodityStyles j
-    q          = queryFromOpts d (reportopts_ opts)
+    q          = _rsQuery $ reportspec_ opts
     ps         = filter (matchesPosting q) $ allPostings j
     mprices    = jpricedirectives j
-    cprices    = map (stylePriceDirectiveExceptPrecision styles) $ concatMap postingsPriceDirectivesFromCosts ps
-    icprices   = map (stylePriceDirectiveExceptPrecision styles) $ concatMap postingsPriceDirectivesFromCosts $ mapAmount invertPrice ps
-    allprices  = mprices ++ ifBoolOpt "costs" cprices ++ ifBoolOpt "inverted-costs" icprices
-  mapM_ (putStrLn . showPriceDirective) $
+    cprices    =
+      map (stylePriceDirectiveExceptPrecision styles) $
+      concatMap postingsPriceDirectivesFromCosts ps
+    rcprices   =
+      map (stylePriceDirectiveExceptPrecision styles) $
+      concatMap (postingsPriceDirectivesFromCosts . postingTransformAmount (mapMixedAmount invertPrice))
+      ps
+    allprices  =
+      mprices
+      ++ ifBoolOpt "infer-market-prices" cprices
+      ++ ifBoolOpt "infer-reverse-prices" rcprices  -- TODO: shouldn't this show reversed P prices also ? valuation will use them
+
+  mapM_ (T.putStrLn . showPriceDirective) $
     sortOn pddate $
     filter (matchesPriceDirective q) $
     allprices
@@ -42,8 +55,8 @@ prices opts j = do
     ifBoolOpt opt | boolopt opt $ rawopts_ opts = id
                   | otherwise = const []
 
-showPriceDirective :: PriceDirective -> String
-showPriceDirective mp = unwords ["P", show $ pddate mp, T.unpack . quoteCommoditySymbolIfNeeded $ pdcommodity mp, showAmountWithZeroCommodity $ pdamount mp]
+showPriceDirective :: PriceDirective -> T.Text
+showPriceDirective mp = T.unwords ["P", T.pack . show $ pddate mp, quoteCommoditySymbolIfNeeded $ pdcommodity mp, wbToText . showAmountB noColour{displayZeroCommodity=True} $ pdamount mp]
 
 divideAmount' :: Quantity -> Amount -> Amount
 divideAmount' n a = a' where
@@ -66,21 +79,23 @@ invertPrice a =
             a { aprice = Just $ TotalPrice pa' } where
                 pa' = ((1 / aquantity a) `divideAmount` pa) { aprice = Nothing }
         Just (TotalPrice pa) ->
-            a { aquantity = aquantity pa * signum (aquantity a), acommodity = acommodity pa, aprice = Just $ TotalPrice pa' } where
+            a { aquantity = aquantity pa * nonZeroSignum (aquantity a), acommodity = acommodity pa, aprice = Just $ TotalPrice pa' } where
                 pa' = pa { aquantity = abs $ aquantity a, acommodity = acommodity a, aprice = Nothing, astyle = astyle a }
+  where
+    nonZeroSignum x = if x < 0 then -1 else 1
 
 postingsPriceDirectivesFromCosts :: Posting -> [PriceDirective]
-postingsPriceDirectivesFromCosts p = mapMaybe (amountPriceDirectiveFromCost date) . amounts $ pamount p  where
-   date = fromMaybe (tdate . fromJust $ ptransaction p) $ pdate p
+postingsPriceDirectivesFromCosts p = mapMaybe (amountPriceDirectiveFromCost date) . amountsRaw $ pamount p
+  where date = fromMaybe (tdate . fromJust $ ptransaction p) $ pdate p
 
 amountPriceDirectiveFromCost :: Day -> Amount -> Maybe PriceDirective
 amountPriceDirectiveFromCost d a =
     case aprice a of
-        Nothing -> Nothing
         Just (UnitPrice pa) -> Just
             PriceDirective { pddate = d, pdcommodity = acommodity a, pdamount = pa }
-        Just (TotalPrice pa) -> Just
+        Just (TotalPrice pa) | aquantity a /= 0 -> Just
             PriceDirective { pddate = d, pdcommodity = acommodity a, pdamount = abs (aquantity a) `divideAmount'` pa }
+        _ -> Nothing
 
 -- | Given a map of standard amount display styles, apply the
 -- appropriate one, if any, to this price directive's amount.
@@ -91,8 +106,3 @@ stylePriceDirectiveExceptPrecision styles pd@PriceDirective{pdamount=a} =
 
 allPostings :: Journal -> [Posting]
 allPostings = concatMap tpostings . jtxns
-
-mapAmount :: (Amount -> Amount) -> [Posting] -> [Posting]
-mapAmount f = map pf where
-    pf p = p { pamount = mf (pamount p) }
-    mf = mixed . map f . amounts
