@@ -1,6 +1,8 @@
-{-# LANGUAGE OverloadedStrings, LambdaCase, DeriveTraversable, ViewPatterns #-}
-{-# LANGUAGE CPP #-}
-{-# LANGUAGE TemplateHaskell #-}
+{-# LANGUAGE DeriveTraversable #-}
+{-# LANGUAGE LambdaCase        #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE TemplateHaskell   #-}
+{-# LANGUAGE ViewPatterns      #-}
 
 module Hledger.Cli.Commands.Rewrite (
   rewritemode
@@ -8,12 +10,11 @@ module Hledger.Cli.Commands.Rewrite (
 )
 where
 
-#if !(MIN_VERSION_base(4,11,0))
-import Control.Monad.Writer
-#endif
 import Data.Functor.Identity
 import Data.List (sortOn, foldl')
+import Data.Text (Text)
 import qualified Data.Text as T
+import qualified Data.Text.IO as T
 import Hledger
 import Hledger.Cli.CliOptions
 import Hledger.Cli.Commands.Print
@@ -36,25 +37,25 @@ rewritemode = hledgerCommandMode
 -- TODO interpolating match groups in replacement
 -- TODO allow using this on unbalanced entries, eg to rewrite while editing
 
-rewrite opts@CliOpts{rawopts_=rawopts,reportopts_=ropts} j@Journal{jtxns=ts} = do
+rewrite opts@CliOpts{rawopts_=rawopts,reportspec_=rspec} j@Journal{jtxns=ts} = do
   -- rewrite matched transactions
-  d <- getCurrentDay
+  let today = _rsDay rspec
   let modifiers = transactionModifierFromOpts opts : jtxnmodifiers j
-  let j' = j{jtxns=either error' id $ modifyTransactions d modifiers ts}  -- PARTIAL:
+  let j' = j{jtxns=either error' id $ modifyTransactions (journalAccountType j) (journalInheritedAccountTags j) mempty today modifiers ts}  -- PARTIAL:
   -- run the print command, showing all transactions, or show diffs
-  printOrDiff rawopts opts{reportopts_=ropts{query_=""}} j j'
+  printOrDiff rawopts opts{reportspec_=rspec{_rsQuery=Any}} j j'
 
 -- | Build a 'TransactionModifier' from any query arguments and --add-posting flags
 -- provided on the command line, or throw a parse error.
 transactionModifierFromOpts :: CliOpts -> TransactionModifier
-transactionModifierFromOpts CliOpts{rawopts_=rawopts,reportopts_=ropts} =
-  TransactionModifier{tmquerytxt=q, tmpostingrules=ps}
+transactionModifierFromOpts CliOpts{rawopts_=rawopts} =
+    TransactionModifier{tmquerytxt=q, tmpostingrules=ps}
   where
-    q = T.pack $ query_ ropts
+    q = T.pack . unwords . map quoteIfNeeded $ listofstringopt "args" rawopts
     ps = map (parseposting . T.pack) $ listofstringopt "add-posting" rawopts
     parseposting t = either (error' . errorBundlePretty) id ep  -- PARTIAL:
       where
-        ep = runIdentity (runJournalParser (postingp Nothing <* eof) t')
+        ep = runIdentity (runJournalParser (tmpostingrulep Nothing <* eof) t')
         t' = " " <> t <> "\n" -- inject space and newline for proper parsing
 
 printOrDiff :: RawOpts -> (CliOpts -> Journal -> Journal -> IO ())
@@ -65,9 +66,9 @@ printOrDiff opts
 diffOutput :: Journal -> Journal -> IO ()
 diffOutput j j' = do
     let changed = [(originalTransaction t, originalTransaction t') | (t, t') <- zip (jtxns j) (jtxns j'), t /= t']
-    putStr $ renderPatch $ map (uncurry $ diffTxn j) changed
+    T.putStr $ renderPatch $ map (uncurry $ diffTxn j) changed
 
-type Chunk = (GenericSourcePos, [DiffLine String])
+type Chunk = (SourcePos, [DiffLine Text])
 
 -- XXX doctests, update needed:
 -- >>> putStr $ renderPatch [(GenericSourcePos "a" 1 1, [D.First "x", D.Second "y"])]
@@ -95,17 +96,17 @@ type Chunk = (GenericSourcePos, [DiffLine String])
 -- @@ -5,0 +5,1 @@
 -- +z
 -- | Render list of changed lines as a unified diff
-renderPatch :: [Chunk] -> String
+renderPatch :: [Chunk] -> Text
 renderPatch = go Nothing . sortOn fst where
     go _ [] = ""
-    go Nothing cs@((sourceFilePath -> fp, _):_) = fileHeader fp ++ go (Just (fp, 0)) cs
-    go (Just (fp, _)) cs@((sourceFilePath -> fp', _):_) | fp /= fp' = go Nothing cs
-    go (Just (fp, offs)) ((sourceFirstLine -> lineno, diffs):cs) = chunkHeader ++ chunk ++ go (Just (fp, offs + adds - dels)) cs
+    go Nothing cs@((SourcePos fp _ _, _):_) = fileHeader fp <> go (Just (fp, 0)) cs
+    go (Just (fp, _)) cs@((SourcePos fp' _ _, _):_) | fp /= fp' = go Nothing cs
+    go (Just (fp, offs)) ((SourcePos _ lineno _, diffs):cs) = chunkHeader <> chunk <> go (Just (fp, offs + adds - dels)) cs
         where
-            chunkHeader = printf "@@ -%d,%d +%d,%d @@\n" lineno dels (lineno+offs) adds where
+            chunkHeader = T.pack $ printf "@@ -%d,%d +%d,%d @@\n" (unPos lineno) dels (unPos lineno+offs) adds
             (dels, adds) = foldl' countDiff (0, 0) diffs
-            chunk = concatMap renderLine diffs
-    fileHeader fp = printf "--- %s\n+++ %s\n" fp fp
+            chunk = foldMap renderLine diffs
+    fileHeader fp = "--- " <> T.pack fp <> "\n+++ " <> T.pack fp <> "\n"
 
     countDiff (dels, adds) = \case
         Del _  -> (dels + 1, adds)
@@ -113,30 +114,29 @@ renderPatch = go Nothing . sortOn fst where
         Ctx _ -> (dels + 1, adds + 1)
 
     renderLine = \case
-        Del s -> '-' : s ++ "\n"
-        Add s -> '+' : s ++ "\n"
-        Ctx s -> ' ' : s ++ "\n"
+        Del s -> "-" <> s <> "\n"
+        Add s -> "+" <> s <> "\n"
+        Ctx s -> " " <> s <> "\n"
 
 diffTxn :: Journal -> Transaction -> Transaction -> Chunk
-diffTxn j t t' =
-        case tsourcepos t of
-            GenericSourcePos fp lineno _ -> (GenericSourcePos fp (lineno+1) 1, diffs) where
-                -- TODO: use range and produce two chunks: one removes part of
-                --       original file, other adds transaction to new file with
-                --       suffix .ledger (generated). I.e. move transaction from one file to another.
-                diffs :: [DiffLine String]
-                diffs = concat . map (traverse showPostingLines . mapDiff) $ D.getDiff (tpostings t) (tpostings t')
-            pos@(JournalSourcePos fp (line, line')) -> (pos, diffs) where
-                -- We do diff for original lines vs generated ones. Often leads
-                -- to big diff because of re-format effect.
-                diffs :: [DiffLine String]
-                diffs = map mapDiff $ D.getDiff source changed'
-                source | Just contents <- lookup fp $ jfiles j = map T.unpack . drop (line-1) . take line' $ T.lines contents
-                       | otherwise = []
-                changed = lines $ showTransaction t'
-                changed' | null changed = changed
-                         | null $ last changed = init changed
-                         | otherwise = changed
+diffTxn j t t' = case tsourcepos t of
+    (pos1@(SourcePos fp line col), pos2) | pos1 == pos2 -> (SourcePos fp (line <> mkPos 1) col, diffs) where
+        -- TODO: use range and produce two chunks: one removes part of
+        --       original file, other adds transaction to new file with
+        --       suffix .ledger (generated). I.e. move transaction from one file to another.
+        diffs :: [DiffLine Text]
+        diffs = concatMap (traverse showPostingLines . mapDiff) $ D.getDiff (tpostings t) (tpostings t')
+    (pos1@(SourcePos fp line _), SourcePos _ line' _) -> (pos1, diffs) where
+        -- We do diff for original lines vs generated ones. Often leads
+        -- to big diff because of re-format effect.
+        diffs :: [DiffLine Text]
+        diffs = map mapDiff $ D.getDiff source changed'
+        source | Just contents <- lookup fp $ jfiles j = drop (unPos line-1) . take (unPos line' - 1) $ T.lines contents
+               | otherwise = []
+        changed = T.lines $ showTransaction t'
+        changed' | null changed = changed
+                 | T.null $ last changed = init changed
+                 | otherwise = changed
 
 data DiffLine a = Del a | Add a | Ctx a
     deriving (Show, Functor, Foldable, Traversable)
