@@ -19,7 +19,7 @@ You can use the command line:
 or ghci:
 
 > $ ghci hledger
-> > j <- readJournalFile def "examples/sample.journal"
+> > Right j <- readJournalFile definputopts "examples/sample.journal"
 > > register [] ["income","expenses"] j
 > 2008/01/01 income               income:salary                   $-1          $-1
 > 2008/06/01 gift                 income:gifts                    $-1          $-2
@@ -30,13 +30,13 @@ or ghci:
 >                   $2  expenses
 >                  $-2  income
 >                   $1  liabilities
-> > l <- myLedger
+> > j <- defaultJournal
 
-See "Hledger.Data.Ledger" for more examples.
+etc.
 
 -}
 
-{-# LANGUAGE QuasiQuotes #-}
+{-# LANGUAGE LambdaCase #-}
 
 module Hledger.Cli.Main where
 
@@ -51,6 +51,7 @@ import System.Process
 import Text.Printf
 
 import Hledger.Cli
+import Data.Time.Clock.POSIX (getPOSIXTime)
 
 
 -- | The overall cmdargs mode describing hledger's command-line options and subcommands.
@@ -96,6 +97,7 @@ mainmode addons = defMode {
 -- | Let's go!
 main :: IO ()
 main = do
+  progstarttime <- getPOSIXTime
 
   -- Choose and run the appropriate internal or external command based
   -- on the raw command-line arguments, cmdarg's interpretation of
@@ -129,7 +131,8 @@ main = do
   let addons = filter (not . (`elem` builtinCommandNames) . dropExtension) addons'
 
   -- parse arguments with cmdargs
-  opts <- argsToCliOpts args addons
+  opts' <- argsToCliOpts args addons
+  let opts = opts'{progstarttime_=progstarttime}
 
   -- select an action and run it.
   let
@@ -138,35 +141,40 @@ main = do
     isExternalCommand    = not (null cmd) && cmd `elem` addons -- probably
     isBadCommand         = not (null rawcmd) && null cmd
     hasVersion           = ("--version" `elem`)
-    hasDetailedVersion   = ("--version+" `elem`)
     printUsage           = putStr $ showModeUsage $ mainmode addons
     badCommandError      = error' ("command "++rawcmd++" is not recognized, run with no command to see a list") >> exitFailure  -- PARTIAL:
     hasHelpFlag args     = any (`elem` args) ["-h","--help"]
+    hasManFlag args      = any (`elem` args) ["--man"]
+    hasInfoFlag args     = any (`elem` args) ["--info"]
     f `orShowHelp` mode
       | hasHelpFlag args = putStr $ showModeUsage mode
+      | hasInfoFlag args = runInfoForTopic "hledger" (headMay $ modeNames mode)
+      | hasManFlag args  = runManForTopic "hledger" (headMay $ modeNames mode)
       | otherwise        = f
+      -- where
+      --   lastdocflag
   dbgIO "processed opts" opts
   dbgIO "command matched" cmd
   dbgIO "isNullCommand" isNullCommand
   dbgIO "isInternalCommand" isInternalCommand
   dbgIO "isExternalCommand" isExternalCommand
   dbgIO "isBadCommand" isBadCommand
-  d <- getCurrentDay
-  dbgIO "period from opts" (period_ $ reportopts_ opts)
-  dbgIO "interval from opts" (interval_ $ reportopts_ opts)
-  dbgIO "query from opts & args" (queryFromOpts d $ reportopts_ opts)
+  dbgIO "period from opts" (period_ . _rsReportOpts $ reportspec_ opts)
+  dbgIO "interval from opts" (interval_ . _rsReportOpts $ reportspec_ opts)
+  dbgIO "query from opts & args" (_rsQuery $ reportspec_ opts)
   let
-    journallesserror = error "journal-less command tried to use the journal"
+    journallesserror = error $ cmd++" tried to read the journal but is not supposed to"
     runHledgerCommand
       -- high priority flags and situations. -h, then --help, then --info are highest priority.
-      | hasHelpFlag argsbeforecmd = dbgIO "" "-h before command, showing general usage" >> printUsage
-      | not (hasHelpFlag argsaftercmd) && (hasVersion argsbeforecmd || (hasVersion argsaftercmd && isInternalCommand))
+      | isNullCommand && hasHelpFlag args = dbgIO "" "-h/--help with no command, showing general help" >> printUsage
+      | isNullCommand && hasInfoFlag args = dbgIO "" "--info with no command, showing general info manual" >> runInfoForTopic "hledger" Nothing
+      | isNullCommand && hasManFlag args  = dbgIO "" "--man with no command, showing general man page" >> runManForTopic "hledger" Nothing
+      | not (isExternalCommand || hasHelpFlag args || hasInfoFlag args || hasManFlag args)
+        && (hasVersion args) --  || (hasVersion argsaftercmd && isInternalCommand))
                                  = putStrLn prognameandversion
-      | not (hasHelpFlag argsaftercmd) && (hasDetailedVersion argsbeforecmd || (hasDetailedVersion argsaftercmd && isInternalCommand))
-                                 = putStrLn prognameanddetailedversion
       -- \| (null externalcmd) && "binary-filename" `inRawOpts` rawopts = putStrLn $ binaryfilename progname
       -- \| "--browse-args" `elem` args     = System.Console.CmdArgs.Helper.execute "cmdargs-browser" mainmode' args >>= (putStr . show)
-      | isNullCommand            = dbgIO "" "no command, showing commands list" >> printCommandsList addons
+      | isNullCommand            = dbgIO "" "no command, showing commands list" >> printCommandsList prognameandversion addons
       | isBadCommand             = badCommandError
 
       -- builtin commands
@@ -176,7 +184,7 @@ main = do
           _ | cmd `elem` ["test","help"] -> cmdaction opts journallesserror
           -- these commands should create the journal if missing
           _ | cmd `elem` ["add","import"] -> do
-            (ensureJournalFileExists =<< (head <$> journalFilePathFromOpts opts))
+            ensureJournalFileExists . head =<< journalFilePathFromOpts opts
             withJournalDo opts (cmdaction opts)
           -- other commands read the journal and should fail if it's missing
           _ -> withJournalDo opts (cmdaction opts)
@@ -185,7 +193,7 @@ main = do
 
       -- addon commands
       | isExternalCommand = do
-          let externalargs = argsbeforecmd ++ filter (not.(=="--")) argsaftercmd
+          let externalargs = argsbeforecmd ++ filter (/="--") argsaftercmd
           let shellcmd = printf "%s-%s %s" progname cmd (unwords' externalargs) :: String
           dbgIO "external command selected" cmd
           dbgIO "external command arguments" (map quoteIfNeeded externalargs)
@@ -226,33 +234,34 @@ moveFlagsAfterCommand args = moveArgs $ ensureDebugHasArg args
     ensureDebugHasArg as =
       case break (=="--debug") as of
        (bs,"--debug":c:cs) | null c || not (all isDigit c) -> bs++"--debug=1":c:cs
-       (bs,"--debug":[])                                   -> bs++"--debug=1":[]
+       (bs,["--debug"])                                    -> bs++["--debug=1"]
        _                                                   -> as
 
     moveArgs args = insertFlagsAfterCommand $ moveArgs' (args, [])
       where
-        -- -h ..., --version ...
-        moveArgs' ((f:a:as), flags)   | isMovableNoArgFlag f                   = moveArgs' (a:as, flags ++ [f])
         -- -f FILE ..., --alias ALIAS ...
         moveArgs' ((f:v:a:as), flags) | isMovableReqArgFlag f, isValue v       = moveArgs' (a:as, flags ++ [f,v])
         -- -fFILE ..., --alias=ALIAS ...
-        moveArgs' ((fv:a:as), flags)  | isMovableReqArgFlagAndValue fv         = moveArgs' (a:as, flags ++ [fv])
+        moveArgs' ((fv:a:as), flags)  | isMovableArgFlagAndValue fv            = moveArgs' (a:as, flags ++ [fv])
         -- -f(missing arg)
         moveArgs' ((f:a:as), flags)   | isMovableReqArgFlag f, not (isValue a) = moveArgs' (a:as, flags ++ [f])
+        -- -h ..., --version ...
+        moveArgs' ((f:a:as), flags)   | isMovableNoArgFlag f                   = moveArgs' (a:as, flags ++ [f])
         -- anything else
         moveArgs' (as, flags) = (as, flags)
 
         insertFlagsAfterCommand ([],           flags) = flags
         insertFlagsAfterCommand (command:args, flags) = [command] ++ flags ++ args
 
-isMovableNoArgFlag a  = "-" `isPrefixOf` a && dropWhile (=='-') a `elem` noargflagstomove
+isMovableNoArgFlag a  = "-" `isPrefixOf` a && dropWhile (=='-') a `elem` optargflagstomove ++ noargflagstomove
 
 isMovableReqArgFlag a = "-" `isPrefixOf` a && dropWhile (=='-') a `elem` reqargflagstomove
 
-isMovableReqArgFlagAndValue ('-':'-':a:as) = case break (== '=') (a:as) of (f:fs,_:_) -> (f:fs) `elem` reqargflagstomove
-                                                                           _          -> False
-isMovableReqArgFlagAndValue ('-':shortflag:_:_) = [shortflag] `elem` reqargflagstomove
-isMovableReqArgFlagAndValue _ = False
+isMovableArgFlagAndValue ('-':'-':a:as) = case break (== '=') (a:as) of
+    (f:fs,_:_) -> (f:fs) `elem` optargflagstomove ++ reqargflagstomove
+    _          -> False
+isMovableArgFlagAndValue ('-':shortflag:_:_) = [shortflag] `elem` reqargflagstomove
+isMovableArgFlagAndValue _ = False
 
 isValue "-"     = True
 isValue ('-':_) = False
@@ -262,4 +271,9 @@ flagstomove = inputflags ++ reportflags ++ helpflags
 noargflagstomove  = concatMap flagNames $ filter ((==FlagNone).flagInfo) flagstomove
 reqargflagstomove = -- filter (/= "debug") $
                     concatMap flagNames $ filter ((==FlagReq ).flagInfo) flagstomove
-
+optargflagstomove = concatMap flagNames $ filter (isFlagOpt   .flagInfo) flagstomove
+  where
+    isFlagOpt = \case
+      FlagOpt     _ -> True
+      FlagOptRare _ -> True
+      _             -> False
